@@ -1,6 +1,6 @@
 ########################################################################
 ##
-## Copyright (C) 2010-2020 The Octave Project Developers
+## Copyright (C) 2010-2024 The Octave Project Developers
 ##
 ## See the file COPYRIGHT.md in the top-level directory of this
 ## distribution or <https://octave.org/copyright/>.
@@ -34,7 +34,7 @@
 ## @deftypefnx {} {} legend ("@var{command}")
 ## @deftypefnx {} {} legend (@var{hax}, @dots{})
 ## @deftypefnx {} {} legend (@var{hleg}, @dots{})
-## @deftypefnx {} {@var{hleg, hplt} =} legend (@dots{})
+## @deftypefnx {} {@var{hleg} =} legend (@dots{})
 ##
 ## Display a legend for the current axes using the specified strings as labels.
 ##
@@ -195,10 +195,18 @@ function [hleg, hleg_obj, hplot, labels] = legend (varargin)
 
   ## Use the old legend code to handle gnuplot toolkit
   if (strcmp (graphics_toolkit (), "gnuplot"))
-    [hleg, hleg_obj, hplot, labels] = __gnuplot_legend__ (varargin{:});
+    if (nargout > 0)
+      [hleg, hleg_obj, hplot, labels] = __gnuplot_legend__ (varargin{:});
+    else
+      __gnuplot_legend__ (varargin{:});
+    endif
     return;
   endif
 
+  ## FIXME: This function needs to be locked to avoid bug #59439.  Remove this
+  ##        lock once that bug is properly fixed.
+  mlock ();
+
   opts = parse_opts (varargin{:});
 
   hl = opts.legend_handle;
@@ -300,22 +308,22 @@ function [hleg, hleg_obj, hplot, labels] = legend (varargin)
                       "deletefcn", {@reset_cb, hl});
 
     ## Listeners to foreign objects properties are stored for later
-    ## deletion in "delfunction"
+    ## deletion in "reset_cb"
     hax = opts.axes_handles(1);
     hf = ancestor (hax, "figure");
 
     add_safe_listener (hl, hf, "colormap", ...
-                       @() set (hl, "colormap", get (hax, "colormap")));
+                       @(~, ~) set (hl, "colormap", get (hax, "colormap")));
 
     add_safe_listener (hl, hax, "position", {@maybe_update_layout_cb, hl});
     add_safe_listener (hl, hax, "tightinset", ...
-                       @(h) update_layout_cb (get (h, "__legend_handle__")));
+                       @(h, ~) update_layout_cb (get (h, "__legend_handle__")));
     add_safe_listener (hl, hax, "clim", ...
-                       @(hax) set (hl, "clim", get (hax, "clim")));
+                       @(hax, ~) set (hl, "clim", get (hax, "clim")));
     add_safe_listener (hl, hax, "colormap", ...
-                       @(hax) set (hl, "colormap", get (hax, "colormap")));
+                       @(hax, ~) set (hl, "colormap", get (hax, "colormap")));
     add_safe_listener (hl, hax, "fontsize", ...
-                       @(hax) set (hl, "fontsize", 0.9*get (hax, "fontsize")));
+                       @(hax, ~) set (hl, "fontsize", 0.9*get (hax, "fontsize")));
     add_safe_listener (hl, hax, "children", {@legend_autoupdate_cb, hl});
 
     ## Listeners to legend properties
@@ -341,8 +349,8 @@ function [hleg, hleg_obj, hplot, labels] = legend (varargin)
     addlistener (hl, "string", @update_string_cb);
 
     addlistener (hl, "textcolor", ...
-                 @(h) set (findobj (h, "type", "text"), ...
-                           "color", get (hl, "textcolor")));
+                 @(h, ~) set (findobj (h, "type", "text"), ...
+                              "color", get (hl, "textcolor")));
 
     addlistener (hl, "visible", @update_visible_cb);
 
@@ -374,7 +382,7 @@ function [hleg, hleg_obj, hplot, labels] = legend (varargin)
 
 endfunction
 
-function update_box_cb (hl)
+function update_box_cb (hl, ~)
 
   if (strcmp (get (hl, "box"), "on"))
     if (strcmp (get (hl, "color"), "none"))
@@ -400,23 +408,27 @@ function update_location_cb (hl, ~, do_layout = true)
 
 endfunction
 
-function update_edgecolor_cb (hl)
+function update_edgecolor_cb (hl, ~)
 
   ecolor = get (hl, "edgecolor");
   set (hl, "xcolor", ecolor, "ycolor", ecolor);
 
 endfunction
 
-function update_position_cb (hl)
+function update_position_cb (hl, ~)
 
   updating = getappdata (hl, "__updating_layout__");
   if (isempty (updating) || ! updating)
-    set (hl, "location", "none");
+    if (! strcmp (get (hl, "location"), "none"))
+      set (hl, "location", "none");
+    else
+      update_layout_cb (hl);
+    endif
   endif
 
 endfunction
 
-function update_string_cb (hl)
+function update_string_cb (hl, ~)
 
   ## Check that the number of legend item and the number of labels match
   ## before calling update_layout_cb.
@@ -450,7 +462,7 @@ function update_string_cb (hl)
 
 endfunction
 
-function update_visible_cb (hl)
+function update_visible_cb (hl, ~)
 
   location = get (hl, "location");
   if (strcmp (location(end:-1:end-3), "edis"))
@@ -459,7 +471,7 @@ function update_visible_cb (hl)
 
 endfunction
 
-function reset_cb (ht, evt, hl, deletelegend = true)
+function reset_cb (ht, ~, hl, deletelegend = true)
 
   if (ishghandle (hl))
     listeners = getappdata (hl, "__listeners__");
@@ -476,7 +488,7 @@ function reset_cb (ht, evt, hl, deletelegend = true)
 
 endfunction
 
-function delete_legend_cb (hl)
+function delete_legend_cb (hl, ~)
 
   reset_cb ([], [], hl, false);
 
@@ -521,7 +533,7 @@ function addproperties (hl)
 
   addproperty ("numcolumnsmode", hl, "radio", "{auto}|manual");
 
-  addlistener (hl, "numcolumns", @(h) set (h, "numcolumnsmode", "manual"));
+  addlistener (hl, "numcolumns", @(h, ~) set (h, "numcolumnsmode", "manual"));
 
   addproperty ("autoupdate", hl, "radio", "{on}|off");
 
@@ -539,7 +551,7 @@ function addproperties (hl)
 
 endfunction
 
-function maybe_update_layout_cb (h, d, hl)
+function maybe_update_layout_cb (h, ~, hl)
 
   persistent updating = false;
 
@@ -565,7 +577,7 @@ function maybe_update_layout_cb (h, d, hl)
 
 endfunction
 
-function update_numchild_cb (hl)
+function update_numchild_cb (hl, ~)
 
   if (strcmp (get (hl, "autoupdate"), "on"))
 
@@ -583,7 +595,7 @@ function update_numchild_cb (hl)
 
 endfunction
 
-function legend_autoupdate_cb (hax, d, hl)
+function legend_autoupdate_cb (hax, ~, hl)
 
   ## Get all current children including eventual peer plotyy axes children
   try
@@ -613,7 +625,7 @@ function legend_autoupdate_cb (hax, d, hl)
 
     ## FIXME: if the latest child is an hggroup, we cannot label it since this
     ## function is called before the hggroup has been properly populated.
-    persistent valid_types = {"line", "patch", "surface"};
+    persistent valid_types = {"line", "patch", "scatter", "surface"};
     if (! any (strcmp (get (kids, "type"), valid_types)))
       kids = [];
     endif
@@ -723,7 +735,7 @@ function opts = parse_opts (varargin)
 
   ## List plot objects that can be handled
   warn_extra_obj = false;
-  persistent valid_types = {"line", "patch", "surface", "hggroup"};
+  persistent valid_types = {"hggroup", "line", "patch", "scatter", "surface"};
 
   if (nargs > 0 && all (ishghandle (varargin{1})))
 
@@ -886,12 +898,13 @@ function [labels, next_idx] = displayname_or_default (hplots, hl = [])
 endfunction
 
 function update_layout_cb (hl, ~, update_item = false)
+
   updating = getappdata (hl, "__updating_layout__");
   if (! isempty (updating) && updating)
     return;
   endif
 
-  setappdata(hl, "__updating_layout__", true);
+  setappdata (hl, "__updating_layout__", true);
 
   ## Scale limits so that item positions are expressed in points, from
   ## top to bottom and from left to right or reverse depending on textposition
@@ -900,10 +913,10 @@ function update_layout_cb (hl, ~, update_item = false)
 
   unwind_protect
 
-    if (update_item)
-      pos = get (hl, "position")(3:4);
-      set (hl, "xlim",  [0, pos(1)], "ylim",  [0, pos(2)]);
+    pos = get (hl, "position");
 
+    if (update_item)
+      set (hl, "xlim",  [0, pos(3)], "ylim",  [0, pos(4)]);
       textright = strcmp (get (hl, "textposition"), "right");
       set (hl, "ydir", "reverse", ...
                "xdir", ifelse (textright, "normal", "reverse"));
@@ -915,15 +928,38 @@ function update_layout_cb (hl, ~, update_item = false)
       ## Prepare the array of text/icon pairs and update their position
       sz = update_texticon_position (hl, objlist);
     else
-      sz = [diff(get (hl, "xlim")), diff(get (hl, "ylim"))];
+      sz = getappdata (hl, "__item_bouding_box__");
     endif
 
     ## Place the legend
-    update_legend_position (hl, sz);
+    if (! strcmp (get (hl, "location"), "none"))
+      update_legend_position (hl, sz);
+    else
+      ## Custom location: Adapt width and height if necessary.
+      [x0, y0, x1, y1] = deal (0, 0, sz(1), sz(2));
+
+      if (sz(1) > pos(3))
+        pos(3) = sz(1);
+      else
+        dx = pos(3)-sz(1);
+        x0 -= dx/2;
+        x1 += dx/2;
+      endif
+
+      if (sz(2) > pos(4))
+        pos(4) = sz(2);
+      else
+        dy = pos(4)-sz(2);
+        y0 -= dy/2;
+        y1 += dy/2;
+      endif
+
+      set (hl, "position",  pos, "xlim", [x0 x1], "ylim", [y0 y1]);
+    endif
 
   unwind_protect_cleanup
     set (hl, "units", units);
-    setappdata(hl, "__updating_layout__", false);
+    setappdata (hl, "__updating_layout__", false);
   end_unwind_protect
 
 endfunction
@@ -996,7 +1032,7 @@ function [htxt, hicon] = create_item (hl, str, txtpval, hplt)
 
   ## For unknown hggroups use the first child that can be labeled
   persistent known_creators = {"__contour__", "__errplot__", "__quiver__", ...
-                               "__scatter__", "__stem__"};
+                               "__stem__"};
   base_hplt = hplt;
 
   if (strcmp (typ, "hggroup"))
@@ -1006,6 +1042,8 @@ function [htxt, hicon] = create_item (hl, str, txtpval, hplt)
       typ = creator;
       switch (creator)
         case "__contour__"
+          ## Eliminate eventual text objects created by clabel
+          kids(strcmp (get (kids, "type"), "text")) = [];
           hplt = [kids(end), kids(1)];
         case {"__errplot__", "__quiver__", "__stem__"}
           hplt = kids(2:-1:1);
@@ -1032,32 +1070,45 @@ function [htxt, hicon] = create_item (hl, str, txtpval, hplt)
                        "linestyle", "linewidth", ...
                        "marker", "markeredgecolor", ...
                        "markerfacecolor", "markersize"};
+  persistent sprops = {"marker", "markeredgecolor", ...
+                       "markerfacecolor"};
 
   switch (typ)
     case {"line", "__errplot__", "__quiver__", "__stem__"}
 
       ## Main line
       vals = get (hplt(1), lprops);
-      hicon = __go_line__ (hl, [lprops; vals]{:});
+      hicon = __go_line__ (hl, [lprops; vals]{:}, ...
+                           "pickableparts", "all", ...
+                           "buttondownfcn", ...
+                           {@execute_itemhit, hl, hplt, "icon"});
+
       addproperty ("markerxdata", hicon, "double", 0);
       addproperty ("markerydata", hicon, "double", 0);
 
       ## Additional line for the marker
       vals = get (hplt(end), mprops);
       hmarker = __go_line__ (hl, "handlevisibility", "off", ...
-                             "xdata", 0, "ydata", 0, [mprops; vals]{:});
+                             "xdata", 0, "ydata", 0, [mprops; vals]{:}, ...
+                             "pickableparts", "all", ...
+                             "buttondownfcn", ...
+                             {@execute_itemhit, hl, hplt, "icon"});
+      addproperty ("markertruesize", hmarker, "double", NaN);
       update_marker_cb (hmarker);
 
       ## Listeners
       safe_property_link (hplt(1), hicon, lprops);
       safe_property_link (hplt(end), hmarker, mprops);
       addlistener (hicon, "ydata", ...
-                   @(h) set (hmarker, "ydata", get (h, "markerydata")));
+                   @(h, ~) set (hmarker, "ydata", get (h, "markerydata")));
       addlistener (hicon, "xdata", ...
-                   @(h) set (hmarker, "xdata", get (h, "markerxdata")));
-      addlistener (hmarker, "markersize", @update_marker_cb);
+                   @(h, ~) set (hmarker, "xdata", get (h, "markerxdata")));
+      addlistener (hicon, "visible", ...
+                   @(h, ~) set (hmarker, "visible", get (h, "visible")));
+      addlistener (hmarker, "markersize", {@update_marker_cb, true});
+      addlistener (hmarker, "marker", {@update_marker_cb, false});
       add_safe_listener (hl, hplt(1), "beingdeleted",
-                         @() delete ([hicon hmarker]))
+                         @(~, ~) delete ([hicon hmarker]))
       if (! strcmp (typ, "__errplot__"))
         setappdata (hicon, "__creator__", typ);
       else
@@ -1065,7 +1116,7 @@ function [htxt, hicon] = create_item (hl, str, txtpval, hplt)
                     "__format__", get (base_hplt, "format"));
       endif
 
-    case {"patch", "surface", "__scatter__"}
+    case {"patch", "surface"}
 
       vals = get (hplt, pprops);
 
@@ -1076,12 +1127,48 @@ function [htxt, hicon] = create_item (hl, str, txtpval, hplt)
 
       setappdata (hicon, "__creator__", typ);
 
+    case "scatter"
+
+      all_sprops = [sprops, "sizedata", "cdata"];
+
+      vals = get (hplt, all_sprops);
+
+      ## sizedata and cdata may be N-by-1 vectors or N-by-3 (RGB) martrices.
+      ## Take the average for the icon.
+      vals {end-1} = mean (vals {end-1}, 1);
+      vals {end} = mean (vals {end}, 1);
+
+      hicon = __go_scatter__ (hl, [all_sprops; vals]{:}, ...
+                              "pickableparts", "all", ...
+                              "buttondownfcn", ...
+                              {@execute_itemhit, hl, hplt, "icon"});
+
+      ## Simple Listeners
+      safe_property_link (hplt(1), hicon, sprops);
+
+      ## Listener to sizedata
+      lsn = {hplt(1), "sizedata", @(h, ~) set (hicon, "sizedata", ...
+                                               mean (get (h, "sizedata")))};
+      addlistener (lsn{:});
+      addlistener (hicon, "beingdeleted", @(~, ~) dellistener (lsn{:}));
+
+      ## Listener to cdata
+      lsn = {hplt(1), "cdata", @(h, ~) set (hicon, "cdata", ...
+                                            mean (get (h, "cdata"), 1))};
+      addlistener (lsn{:});
+      addlistener (hicon, "beingdeleted", @(~, ~) dellistener (lsn{:}));
+
+      setappdata (hicon, "__creator__", typ);
+
     case "__contour__"
 
       ## Main patch
 
       vals = get (hplt(1), pprops);
-      hicon = __go_patch__ (hl, [pprops; vals]{:});
+      hicon = __go_patch__ (hl, [pprops; vals]{:}, ...
+                            "pickableparts", "all", ...
+                            "buttondownfcn", ...
+                            {@execute_itemhit, hl, hplt, "icon"});
 
       addproperty ("innerxdata", hicon, "any", 0);
       addproperty ("innerydata", hicon, "any", 0);
@@ -1089,36 +1176,60 @@ function [htxt, hicon] = create_item (hl, str, txtpval, hplt)
       ## Additional patch for the inner contour
       vals = get (hplt(end), pprops);
       htmp =  __go_patch__ (hl, "handlevisibility", "off", ...
-                            "xdata", 0, "ydata", 0, [pprops; vals]{:});
+                            "xdata", 0, "ydata", 0, [pprops; vals]{:}, ...
+                            "pickableparts", "all", ...
+                            "buttondownfcn", ...
+                            {@execute_itemhit, hl, hplt, "icon"});
 
       ## Listeners
       safe_property_link (hplt(1), hicon, pprops);
       safe_property_link (hplt(end), htmp, pprops);
       addlistener (hicon, "ydata", ...
-                   @(h) set (htmp, "ydata", get (h, "innerydata")));
+                   @(h, ~) set (htmp, "ydata", get (h, "innerydata")));
       addlistener (hicon, "xdata", ...
-                   @(h) set (htmp, "xdata", get (h, "innerxdata")));
+                   @(h, ~) set (htmp, "xdata", get (h, "innerxdata")));
+      addlistener (hicon, "visible", ...
+                   @(h, ~) set (htmp, "visible", get (h, "visible")));
       add_safe_listener (hl, hplt(1), "beingdeleted",
-                         @() delete ([hicon htmp]))
+                         @(~, ~) delete ([hicon htmp]))
 
       setappdata (hicon, "__creator__", typ);
 
   endswitch
 
-  htxt = __go_text__ (hl, "string", str, txtpval{:});
+  htxt = __go_text__ (hl, txtpval{:}, "string", str, ...
+                      "pickableparts", "all", ...
+                      "buttondownfcn", {@execute_itemhit, hl, hplt, "label"});
+
+  set (htxt, "buttondownfcn", {@execute_itemhit, hl, hplt, "label"});
 
   addproperty ("peer_object", htxt, "double", base_hplt);
   addproperty ("peer_object", hicon, "double", base_hplt);
 
 endfunction
 
+function execute_itemhit (h, ~, hl, hplt, region)
+
+  fcn = get (hl, "itemhitfcn");
+
+  if (! isempty (fcn))
+    evt = struct ("Peer", hplt, "Region", region, ...
+                  "SelectionType", get (gcbf (), "selectiontype"), ...
+                  "Source", hl, "EventName", "ItemHit");
+    fcn (hl, evt)
+  endif
+
+endfunction
+
 function safe_property_link (h1, h2, props)
+
   for ii = 1:numel (props)
     prop = props{ii};
-    lsn = {h1, prop, @(h) set (h2, prop, get (h, prop))};
+    lsn = {h1, prop, @(h, ~) set (h2, prop, get (h, prop))};
     addlistener (lsn{:});
-    addlistener (h2, "beingdeleted", @() dellistener (lsn{:}));
+    addlistener (h2, "beingdeleted", @(~, ~) dellistener (lsn{:}));
   endfor
+
 endfunction
 
 function update_displayname_cb (h, ~, hl)
@@ -1133,16 +1244,55 @@ function update_displayname_cb (h, ~, hl)
     str = {str};
   endif
 
-  str{h == getappdata (hl, "__peer_objects__")} = get (h, "displayname");
-
-  set (hl ,"string", str);
+  idx = h == getappdata (hl, "__peer_objects__");
+  if (any (idx))
+    str{idx} = get (h, "displayname");
+    set (hl ,"string", str);
+  endif
 
 endfunction
 
-function update_marker_cb (h)
+## Enforce maximum size of marker so it doesn't overflow legend key
+function update_marker_cb (h, ~, sz_updated = true)
+  persistent is_updating = false;
 
-  if (get (h, "markersize") > 8)
-    set (h, "markersize", 8);
+  if (is_updating)
+    return;
+  endif
+
+  if (sz_updated)
+    ## Size was changed
+    sz = get (h, "markersize");
+    set (h, "markertruesize", sz);  # store true marker size
+
+    if (sz > 8)
+      is_updating = true;
+
+      mark = get (h, "marker");
+      if (strcmp (mark, '.'))
+        set (h, "markersize", min ([sz, 24]));
+      else
+        set (h, "markersize", 8);
+      endif
+
+      is_updating = false;
+    endif
+
+  else
+    ## Marker style was changed
+    sz = get (h, "markertruesize");
+    if (sz > 8)
+      is_updating = true;
+
+      mark = get (h, "marker");
+      if (strcmp (mark, '.'))
+        set (h, "markersize", min ([sz, 24]));
+      else
+        set (h, "markersize", 8);
+      endif
+
+      is_updating = false;
+    endif
   endif
 
 endfunction
@@ -1159,10 +1309,25 @@ function sz = update_texticon_position (hl, objlist)
   icon_height = 0.7 * get (hl, "fontsize");
   set (hl, "fontunits", units);
 
+  types = get (objlist(:,2), "type");
   ext = get (objlist(:,1), "extent");
   markers = get (objlist(:,2), "marker");
-  markersz = get (objlist(:,2), "markersize");
-  types = get (objlist(:,2), "type");
+
+  is_scatter = strcmp (types, "scatter");
+  if (! any (is_scatter))
+    markersz = get (objlist(:,2), "markersize");
+  elseif (rows (objlist) == 1)
+    markersz = mean (get (objlist(1,2), "sizedata").^0.5);
+  else
+    markersz = cell (rows (objlist), 1);
+    for ii = 1:rows (objlist)
+      if (! is_scatter(ii))
+        markersz{ii} = get (objlist(ii,2), "markersize");
+      else
+        markersz{ii} = mean (get (objlist(ii,2), "sizedata").^0.5);
+      endif
+    endfor
+  endif
 
   ## Simple case of 1 text/icon pair
   nitem = rows (objlist);
@@ -1190,7 +1355,9 @@ function sz = update_texticon_position (hl, objlist)
 
     nrow = ceil (nitem / ncol);
 
-    rowheights = arrayfun (@(idx) max([icon_height; ext(idx:nrow:end, 2)]), ...
+    rowheights = arrayfun (@(idx) max([icon_height;
+                                       ext(idx:nrow:end, 2);
+                                       vertcat(markersz(idx:nrow:end){:})]), ...
                            1:nrow);
     x = hmargin;
     for ii = 1:ncol
@@ -1207,8 +1374,8 @@ function sz = update_texticon_position (hl, objlist)
         endif
 
         ybase = y + hg / 2;
-        y0 = y + hg/2 - icon_height/2 + dx;
-        y1 = y + hg/2 + icon_height/2 - dx;
+        y0 = ybase - max (icon_height, dx)/2 + dx;
+        y1 = ybase + max (icon_height, dx)/2 - dx;
 
         update_icon_position (objlist(iter,2), [x+dx, x+icon_width-dx], ...
                               [y0, y1]);
@@ -1233,14 +1400,15 @@ function sz = update_texticon_position (hl, objlist)
 
     nrow = ceil (nitem / ncol);
 
-    colwidth = arrayfun (@(idx) max(ext(idx:ncol:end, 1)),
+    colwidth = arrayfun (@(idx) max (ext(idx:ncol:end, 1)),
                          1:ncol);
     y = vmargin;
     for ii = 1:nrow
       x = hmargin;
 
       endidx = min (iter+ncol-1, nitem);
-      hg = max ([icon_height; ext(iter:endidx,2)]);
+      hg = max ([icon_height; ext(iter:endidx,2); ...
+                 vertcat(markersz{:})]);
 
       for jj = 1:ncol
         if (iter > nitem)
@@ -1255,8 +1423,8 @@ function sz = update_texticon_position (hl, objlist)
         endif
 
         ybase = y + hg / 2;
-        y0 = y + hg/2 - icon_height/2 + dx;
-        y1 = y + hg/2 + icon_height/2 - dx;
+        y0 = ybase - max (icon_height, dx)/2 + dx;
+        y1 = ybase + max (icon_height, dx)/2 - dx;
 
         update_icon_position (objlist(iter,2), [x+dx, x+icon_width-dx], ...
                               [y0, y1]);
@@ -1274,10 +1442,12 @@ function sz = update_texticon_position (hl, objlist)
   endif
 
   sz = [xmax, ymax];
+  setappdata (hl, "__item_bouding_box__", sz);
 
 endfunction
 
 function update_icon_position (hicon, xdata, ydata)
+
   creator = getappdata (hicon, "__creator__");
   switch (creator)
     case "line"
@@ -1317,12 +1487,19 @@ function update_icon_position (hicon, xdata, ydata)
       elseif (strcmp (fmt, "xerr"))
         xdata = [x0+2, x0+2, x0+2, x1-2, x1-2, x1-2, x1-2];
         ydata = [ym+2, ym-2, ym, ym, ym+2, ym-2, ym];
-      else # "both"
+      elseif (strcmp (fmt, "xyerr"))
         xdata = [x0+2, x0+2, x0+2, x1-2, x1-2, x1-2, x1-2, ...
                  xm, xm, xm-2, xm+2, xm, xm, xm-2, xm+2];
         ydata = [ym+2, ym-2, ym, ym, ym+2, ym-2, ym, ...
                  ym, y0, y0, y0, y0, y1, y1, y1];
+      elseif (strncmp (fmt, "box", 3))
+        xdata = [x0+2, x1-2, x1-2, x0+2, x0+2];
+        ydata = [y0, y0, y1, y1, y0];
+      else
+        xdata = [x0, x1];
+        ydata = [ym, ym];
       endif
+
       set (hicon, "markerxdata", xm, "markerydata", ym, ...
            "xdata", xdata, "ydata", ydata);
 
@@ -1335,13 +1512,14 @@ function update_icon_position (hicon, xdata, ydata)
       ydata = [y0, y0, y0+2, y0, y0-2];
       set (hicon, "markerxdata", x0, "markerydata", y0, ...
            "xdata", xdata, "ydata", ydata);
-    case "__scatter__"
-      set (hicon, "xdata", mean(xdata), "ydata", mean(ydata));
+    case "scatter"
+      set (hicon, "xdata", mean (xdata), "ydata", mean (ydata));
     case "__stem__"
       xdata(2) -= (get (get (hicon, "peer_object"), "markersize") / 2);
       set (hicon, "markerxdata", xdata(2), "markerydata", mean (ydata), ...
            "xdata", xdata, "ydata", [mean(ydata), mean(ydata)]);
   endswitch
+
 endfunction
 
 function pos = boxposition (axpos, pba, pbam, dam)
@@ -1738,7 +1916,7 @@ endfunction
 %!demo
 %! clf;
 %! x = 0:0.1:7;
-%! h = plot (x,sin(x), x,cos(x), x,sin(x.^2/10), x,cos(x.^2/10));
+%! h = plot (x,sin (x), x,cos (x), x,sin (x.^2/10), x,cos (x.^2/10));
 %! title ("Only the sin() objects have keylabels");
 %! legend (h([1, 3]), {"sin (x)", "sin (x^2/10)"}, "location", "southwest");
 
@@ -1938,6 +2116,18 @@ endfunction
 %! legend ("Legend Text");
 %! title ({"Multi-line", "titles", "are *not* a", "problem"});
 
+%!demo  # bug 61814
+%! clf;
+%! data = [ [1:5]' , [5:-1:1]', 2.5*ones(5,1) ];
+%! hp = plot (data);
+%! set (hp(1), "marker", 'x', "markersize", 15);
+%! set (hp(2), "marker", 'o', "markersize", 30);
+%! set (hp(3), "marker", '.', "markersize", 30);
+%! legend ({"data1", "data2", "data3"}, "location", "north");
+%! set (hp(2), "marker", '.');
+%! set (hp(3), "marker", 'o');
+%! title ("Marker sizes do not overflow legend box");
+
 ## Test input validation
 %!test
 %! hf = figure ("visible", "off");
