@@ -1,4 +1,9 @@
-## Copyright (C) 2005-2018 David Bateman
+########################################################################
+##
+## Copyright (C) 2005-2024 The Octave Project Developers
+##
+## See the file COPYRIGHT.md in the top-level directory of this
+## distribution or <https://octave.org/copyright/>.
 ##
 ## This file is part of Octave.
 ##
@@ -15,10 +20,12 @@
 ## You should have received a copy of the GNU General Public License
 ## along with Octave; see the file COPYING.  If not, see
 ## <https://www.gnu.org/licenses/>.
+##
+########################################################################
 
 ## -*- texinfo -*-
-## @deftypefn  {} {} __run_test_suite__ (@var{fcndirs}, @var{fixedtestdirs})
-## @deftypefnx {} {} __run_test_suite__ (@var{fcndirs}, @var{fixedtestdirs}, @var{topsrcdir}, @var{topbuilddir})
+## @deftypefn  {} {[@var{pass}, @var{fail}, @var{xfail}, @var{xbug}, @var{skip}, @var{rtskip}, @var{regress}] =} __run_test_suite__ (@var{fcndirs}, @var{fixedtestdirs})
+## @deftypefnx {} {[@var{pass}, @var{fail}, @var{xfail}, @var{xbug}, @var{skip}, @var{rtskip}, @var{regress}] =} __run_test_suite__ (@var{fcndirs}, @var{fixedtestdirs}, @var{topsrcdir}, @var{topbuilddir})
 ## Undocumented internal function.
 ## @end deftypefn
 
@@ -35,6 +42,7 @@ function [pass, fail, xfail, xbug, skip, rtskip, regress] = __run_test_suite__ (
   endif
   files_with_no_tests = {};
   files_with_tests = {};
+  summary_failure_info = struct ([]);
   ## FIXME: These names don't really make sense if we are running
   ##        tests for an installed copy of Octave.
   if (isempty (topsrcdir))
@@ -45,12 +53,15 @@ function [pass, fail, xfail, xbug, skip, rtskip, regress] = __run_test_suite__ (
   endif
 
   pso = page_screen_output ();
+  orig_wquiet = warning ("query", "quiet");
   orig_wstate = warning ();
+  orig_mfile_encoding = mfile_encoding ("utf-8");
   logfile = make_absolute_filename ("fntests.log");
   unwind_protect
     page_screen_output (false);
     warning ("on", "quiet");
     warning ("off", "Octave:deprecated-function");
+    warning ("off", "Octave:legacy-function");
     nfail = dp = dn = dxf = dxb = dsk = drtsk = drgrs = 0;
     try
       fid = fopen (logfile, "wt");
@@ -59,8 +70,9 @@ function [pass, fail, xfail, xbug, skip, rtskip, regress] = __run_test_suite__ (
       endif
       test ("", "explain", fid);
       puts ("\nIntegrated test scripts:\n\n");
+      printf ("%101s\n", "[ CPU    /  CLOCK ]");
       for i = 1:length (fcndirs)
-        [p, n, xf, xb, sk, rtsk, rgrs] = run_test_script (fid, fcndirs{i});
+        [p, n, xf, xb, sk, rtsk, rgrs] = run_test_dir (fid, fcndirs{i}, false);
         dp += p;
         dn += n;
         dxf += xf;
@@ -71,7 +83,7 @@ function [pass, fail, xfail, xbug, skip, rtskip, regress] = __run_test_suite__ (
       endfor
       puts ("\nFixed test scripts:\n\n");
       for i = 1:length (fixedtestdirs)
-        [p, n, xf, xb, sk, rtsk, rgrs] = run_test_dir (fid, fixedtestdirs{i});
+        [p, n, xf, xb, sk, rtsk, rgrs] = run_test_dir (fid, fixedtestdirs{i}, true);
         dp += p;
         dn += n;
         dxf += xf;
@@ -80,6 +92,14 @@ function [pass, fail, xfail, xbug, skip, rtskip, regress] = __run_test_suite__ (
         drtsk += rtsk;
         drgrs += rgrs;
       endfor
+      if (! isempty (summary_failure_info))
+        puts ("\nFailure Summary:\n\n");
+        for i = 1:numel (summary_failure_info)
+          info = summary_failure_info(i);
+          print_test_file_name (info.name);
+          print_pass_fail (info);
+        endfor
+      endif
       puts ("\nSummary:\n\n");
       nfail = dn - dp - dxf - dxb - drgrs;
       printf ("  %-30s %6d\n", "PASS", dp);
@@ -116,10 +136,10 @@ function [pass, fail, xfail, xbug, skip, rtskip, regress] = __run_test_suite__ (
         puts ("indicating which dependencies were not found.\n");
       endif
 
-      ## Weed out deprecated and private functions
-      weed_idx = cellfun (@isempty, regexp (files_with_tests, '\<deprecated\>|\<private\>', 'once'));
+      ## Weed out deprecated, legacy, and private functions
+      weed_idx = cellfun (@isempty, regexp (files_with_tests, '\<deprecated\>|\<legacy\>|\<private\>', 'once'));
       files_with_tests = files_with_tests(weed_idx);
-      weed_idx = cellfun (@isempty, regexp (files_with_no_tests, '\<deprecated\>|\<private\>', 'once'));
+      weed_idx = cellfun (@isempty, regexp (files_with_no_tests, '\<deprecated\>|\<legacy\>|\<private\>', 'once'));
       files_with_no_tests = files_with_no_tests(weed_idx);
 
       report_files_with_no_tests (files_with_tests, files_with_no_tests, ".m");
@@ -136,6 +156,8 @@ function [pass, fail, xfail, xbug, skip, rtskip, regress] = __run_test_suite__ (
   unwind_protect_cleanup
     warning ("off", "all");
     warning (orig_wstate);
+    warning (orig_wquiet.state, "quiet");
+    mfile_encoding (orig_mfile_encoding);
     page_screen_output (pso);
   end_unwind_protect
 
@@ -149,16 +171,14 @@ function [pass, fail, xfail, xbug, skip, rtskip, regress] = __run_test_suite__ (
     regress = drgrs;
   endif
 
-
-  function [dp, dn, dxf, dxb, dsk, drtsk, drgrs] = run_test_dir (fid, d)
+  function [dp, dn, dxf, dxb, dsk, drtsk, drgrs] = run_test_dir (fid, d, is_fixed = false)
 
     lst = dir (d);
     dp = dn = dxf = dxb = dsk = drtsk = drgrs = 0;
     for i = 1:length (lst)
       nm = lst(i).name;
-      if (lst(i).isdir
-          && nm(1) != "." && ! strcmp (nm, "private") && nm(1) != "@")
-        [p, n, xf, xb, sk, rtsk, rgrs] = run_test_dir (fid, [d, filesep, nm]);
+      if (lst(i).isdir && nm(1) != "." && ! strcmp (nm, "private"))
+        [p, n, xf, xb, sk, rtsk, rgrs] = run_test_dir (fid, [d, filesep, nm], is_fixed);
         dp += p;
         dn += n;
         dxf += xf;
@@ -171,27 +191,55 @@ function [pass, fail, xfail, xbug, skip, rtskip, regress] = __run_test_suite__ (
 
     saved_dir = pwd ();
     unwind_protect
-      cd (d);
+      [dnm, fnm] = fileparts (d);
+      if (fnm(1) != "@")
+        cd (d);
+      endif
       for i = 1:length (lst)
         nm = lst(i).name;
-        if (length (nm) > 4 && strcmpi (nm((end-3):end), ".tst"))
-          p = n = xf = xb = sk = rtsk = 0;
+        ## Ignore hidden files
+        if (nm(1) == '.')
+          continue;
+        endif
+        if ((! is_fixed && length (nm) > 2 && strcmpi (nm((end-1):end), ".m"))
+            || (! is_fixed && length (nm) > 4 && strcmpi (nm((end-3):end), "-tst"))
+            || (is_fixed && length (nm) > 4 && strcmpi (nm((end-3):end), ".tst")))
+          p = n = xf = xb = sk = rtsk = rgrs = 0;
           ffnm = fullfile (d, nm);
+          if (! isfile (ffnm))
+            continue;
+          endif
+
+          ## Only run if contains %!test, %!assert, %!error, %!fail, or %!warning
           if (has_tests (ffnm))
-            print_test_file_name (nm);
-            [p, n, xf, xb, sk, rtsk, rgrs] = test (nm, "quiet", fid);
-            print_pass_fail (p, n, xf, xb, sk, rtsk, rgrs);
+            tmp = reduce_test_file_name (ffnm, topbuilddir, topsrcdir);
+            print_test_file_name (tmp);
+            cpu_tm = cputime ();
+            tic ();
+            [p, n, xf, xb, sk, rtsk, rgrs] = test (ffnm, "quiet", fid);
+            clock_tm = toc ();
+            cpu_tm = cputime () - cpu_tm;
+            print_pass_fail (p, n, xf, xb, sk, rtsk, rgrs, cpu_tm, clock_tm);
+            dp += p;
+            dn += n;
+            dxf += xf;
+            dxb += xb;
+            dsk += sk;
+            drtsk += rtsk;
+            drgrs += rgrs;
             files_with_tests(end+1) = ffnm;
+            if (n - p > 0)
+              ## Save info for summary if there were any failed
+              ## tests for this file.
+              failure_info = struct ("name", tmp, "pass", p, "ntst", n,
+                                     "xf", xf, "xb", xb, "rgrs", rgrs);
+              summary_failure_info(end+1) = failure_info;
+            endif
           else
+            ## To reduce the list length, only mark .cc files that contain
+            ## DEFUN definitions.
             files_with_no_tests(end+1) = ffnm;
           endif
-          dp += p;
-          dn += n;
-          dxf += xf;
-          dxb += xb;
-          dsk += sk;
-          drtsk += rtsk;
-          drgrs += rgrs;
         endif
       endfor
     unwind_protect_cleanup
@@ -200,90 +248,59 @@ function [pass, fail, xfail, xbug, skip, rtskip, regress] = __run_test_suite__ (
 
   endfunction
 
-  function [dp, dn, dxf, dxb, dsk, drtsk, drgrs] = run_test_script (fid, d)
-
-    lst = dir (d);
-    dp = dn = dxf = dxb = dsk = drtsk = drgrs = 0;
-    for i = 1:length (lst)
-      nm = lst(i).name;
-      if (lst(i).isdir && nm(1) != ".")
-        [p, n, xf, xb, sk, rtsk, rgrs] = run_test_script (fid, [d, filesep, nm]);
-        dp += p;
-        dn += n;
-        dxf += xf;
-        dxb += xb;
-        dsk += sk;
-        drtsk += rtsk;
-        drgrs += rgrs;
-      endif
-    endfor
-
-    for i = 1:length (lst)
-      nm = lst(i).name;
-      ## Ignore hidden files
-      if (nm(1) == '.')
-        continue
-      endif
-      f = fullfile (d, nm);
-      if ((length (nm) > 2 && strcmpi (nm((end-1):end), ".m"))
-          || (length (nm) > 4
-              && (   strcmpi (nm((end-3):end), "-tst")
-                  || strcmpi (nm((end-3):end), ".tst"))))
-        p = n = xf = xb = 0;
-        ## Only run if contains %!test, %!assert, %!error, %!fail, or %!warning
-        if (has_tests (f))
-          tmp = reduce_test_file_name (f, topbuilddir, topsrcdir);
-          print_test_file_name (tmp);
-          [p, n, xf, xb, sk, rtsk, rgrs] = test (f, "quiet", fid);
-          print_pass_fail (p, n, xf, xb, sk, rtsk, rgrs);
-          dp += p;
-          dn += n;
-          dxf += xf;
-          dxb += xb;
-          dsk += sk;
-          drtsk += rtsk;
-          drgrs += rgrs;
-          files_with_tests(end+1) = f;
-        else
-          ## To reduce the list length, only mark .cc files that contain
-          ## DEFUN definitions.
-          files_with_no_tests(end+1) = f;
-        endif
-      endif
-    endfor
-    ##  printf("%s%s -> passes %d of %d tests\n", ident, d, dp, dn);
-
-  endfunction
-
 endfunction
 
 function print_test_file_name (nm)
-  filler = repmat (".", 1, 60-length (nm));
+
+  nmlen = numel (nm);
+  filler = repmat (".", 1, 63-nmlen);
+  if (nmlen > 63)
+    nm = ["..", nm(nmlen-60:end)];
+  endif
   printf ("  %s %s", nm, filler);
+
 endfunction
 
-function print_pass_fail (p, n, xf, xb, sk, rtsk, rgrs)
+function fail_info = print_pass_fail (p, n, xf, xb, sk, rtsk, rgrs, cpu_tm, clock_tm)
+
+  if (nargin == 1)
+    ## The summary info struct just contains info about failures, not
+    ## skipped tests or timings.
+    info = p;
+    p = info.pass;
+    n = info.ntst;
+    xf = info.xf;
+    xb = info.xb;
+    sk = 0;
+    rtsk = 0;
+    rgrs = info.rgrs;
+    cpu_tm = 0;
+    clock_tm = 0;
+  endif
 
   if ((n + sk + rtsk + rgrs) > 0)
-    printf (" PASS   %4d/%-4d", p, n);
+    printf (" pass %4d/%-4d", p, n);
+    if (cpu_tm != 0 || clock_tm != 0)
+      printf (" [%6.3fs / %6.3fs]", cpu_tm, clock_tm);
+    endif
     nfail = n - p - xf - xb - rgrs;
     if (nfail > 0)
-      printf ("\n%71s %3d", "FAIL ", nfail);
+      printf ("\n%72s %3d", "FAIL ", nfail);
     endif
     if (rgrs > 0)
-      printf ("\n%71s %3d", "REGRESSION", rgrs);
+      printf ("\n%72s %3d", "REGRESSION", rgrs);
     endif
     if (sk > 0)
-      printf ("\n%71s %3d", "(missing feature) SKIP ", sk);
+      printf ("\n%72s %3d", "(missing feature) SKIP ", sk);
     endif
     if (rtsk > 0)
-      printf ("\n%71s %3d", "(run-time condition) SKIP ", rtsk);
+      printf ("\n%72s %3d", "(run-time condition) SKIP ", rtsk);
     endif
     if (xb > 0)
-      printf ("\n%71s %3d", "(reported bug) XFAIL", xb);
+      printf ("\n%72s %3d", "(reported bug) XFAIL", xb);
     endif
     if (xf > 0)
-      printf ("\n%71s %3d", "(expected failure) XFAIL", xf);
+      printf ("\n%72s %3d", "(expected failure) XFAIL", xf);
     endif
   endif
   puts ("\n");
@@ -296,11 +313,12 @@ function retval = reduce_test_file_name (nm, builddir, srcdir)
   ## of the likely root directory prefixes.
 
   prefix = { builddir, fullfile(builddir, "scripts"), ...
-             srcdir, fullfile(srcdir, "scripts") };
+             srcdir, fullfile(srcdir, "scripts"), ...
+             srcdir, fullfile(srcdir, "test") };
 
   retval = nm;
 
-  for i = 1:length (prefix)
+  for i = 1:numel (prefix)
     tmp = strrep (nm, [prefix{i}, filesep], "");
     if (length (tmp) < length (retval))
       retval = tmp;
@@ -349,11 +367,13 @@ function n = num_elts_matching_pattern (lst, pat)
 endfunction
 
 function report_files_with_no_tests (with, without, typ)
+
   pat = ['\' typ "$"];
   n_with = num_elts_matching_pattern (with, pat);
   n_without = num_elts_matching_pattern (without, pat);
   n_tot = n_with + n_without;
   printf ("\n%d (of %d) %s files have no tests.\n", n_without, n_tot, typ);
+
 endfunction
 
 
